#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const commander_1 = __importDefault(require("commander"));
const chalk_1 = __importDefault(require("chalk"));
const open_1 = __importDefault(require("open"));
const generate_1 = require("./generate");
const fs_1 = require("fs");
// Match stack name after first forward slash
const ARN_STACK_NAME_REGEX = /[^\/]*\/([^\/]+)/;
const STACK_PREFIX = 'stack:';
const LINKS = [];
const NODES = new Set();
commander_1.default
    .version('1.0.0')
    .description('Visualize cross-stack references in your AWS CloudFormation stacks.')
    .option('-p, --profile <string>', 'use a specific profile from your credential file.', 'default')
    .option('-r, --region <string>', 'the region to use.', 'eu-west-1')
    .option('-b, --browser <string>', 'the browser to use.', 'firefox')
    .parse(process.argv);
// Debug
commander_1.default.profile = 'prod';
// Configure AWS SDK
aws_sdk_1.default.config.update({
    apiVersion: '2010-05-15',
    credentials: new aws_sdk_1.default.SharedIniFileCredentials({ profile: commander_1.default.profile }),
    region: commander_1.default.region
});
const log = (str, nl = true) => {
    if (nl) {
        console.log(str);
    }
    else {
        process.stdout.write(str);
    }
};
(() => __awaiter(this, void 0, void 0, function* () {
    const CF = new aws_sdk_1.default.CloudFormation();
    try {
        // Download all exports
        log(chalk_1.default.grey('Download all exports... '), false);
        const { Exports } = yield CF.listExports().promise();
        log(chalk_1.default.keyword('green').bold('OK!'));
        const jobs = Exports.map(({ ExportingStackId, Name }) => __awaiter(this, void 0, void 0, function* () {
            // Which stack?
            const source = STACK_PREFIX + ExportingStackId.match(ARN_STACK_NAME_REGEX).pop();
            // What export?
            const target = Name;
            LINKS.push({
                source,
                target,
                value: 1
            });
            NODES.add(source).add(target);
            try {
                // Download all stacks who import current export
                const { Imports } = yield CF.listImports({ ExportName: target }).promise();
                Imports.forEach(t => {
                    t = STACK_PREFIX + t;
                    LINKS.push({
                        source: target,
                        target: t,
                        value: 1
                    });
                    NODES.add(t);
                });
            }
            catch (e) { }
        }));
        log(chalk_1.default.grey(`Found ${Exports.length} exports. Inspect imports... `), false);
        yield Promise.all(jobs);
        log(chalk_1.default.keyword('green').bold('OK!'));
        log(chalk_1.default.grey(`Opening default web browser... `), false);
        const html = generate_1.generate({
            links: LINKS,
            nodes: [...NODES]
        });
        fs_1.writeFileSync('./index.html', html);
        yield open_1.default('./index.html', { wait: true, app: commander_1.default.browser });
        log(chalk_1.default.keyword('green').bold('OK!'));
    }
    catch (e) {
        log(chalk_1.default.keyword('red').bold(`FAIL! (${e.message})`));
    }
}))();
//# sourceMappingURL=index.js.map